use std::{error::Error, fs::OpenOptions, str::FromStr};
use clap::{Args, Parser, Subcommand};
use shoebill::{BundleDefinition, BundleType, Bundle};

pub mod shoebill {
    tonic::include_proto!("shoebill");
}

impl FromStr for BundleType {

    type Err = ();

    fn from_str(input: &str) -> Result<BundleType, Self::Err> {
        match input {
            "package"  => Ok(BundleType::TypePackage),
            "library"  => Ok(BundleType::TypeLibrary),
            _      => Err(()),
        }
    }
}

#[derive(Parser)]
#[command(author, version, about, long_about = None)]
#[command(propagate_version = true)]
struct Cli {
    #[command(subcommand)]
    command: Commands,
}


#[derive(Subcommand)]
enum Commands {
    /// Adds files to myapp
    Install(InstallArgs),
}


#[derive(Args)]
struct InstallArgs {
    #[arg(short, long)]
    path: String,
}

fn define_bundle(path: String) -> Result<BundleDefinition, Box<dyn Error>> {
    let config_res: std::result::Result<BundleDefinition, _>;
    let f = OpenOptions::new().write(false).read(true).open(path);
    let f = match f {
        Ok(file) => file,
        Err(err) => {
            return Err(err.into());
        }
    };
    config_res = serde_yaml::from_reader(f);
    match config_res {
        Ok(config) => Ok(config),
        Err(err) => Err(err.into()),
    }
}

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    // First iteration ------------------------------------------------
    // Install without repository, just a path to a file
    // ----------------------------------------------------------------
    // Derive from yaml file
    let cli = Cli::parse();

    // You can check for the existence of subcommands, and if found use their
    // matches just as you would the top level cmd

    let definition: BundleDefinition;
    match &cli.command {
        Commands::Install(args) => {
            match  define_bundle(args.path.clone()) {
                Ok(def) => definition = def,
                Err(err) => panic!("{}", err),
            }
        }
    }

    let r#type = BundleType::from_str(definition.clone().spec.unwrap().r#type.as_str()).unwrap();
    let bundle = Bundle{definition: Some(definition.clone()), r#type: r#type.into()};
    println!("{:?}", bundle);


    Ok(())
    // let mut client = RepoClient::connect("http://[::1]:50051").await?;
    //
    // let request = tonic::Request::new(Bundle {
    // name: "Tonic".into(),
    // });
    //
    // let response = client.download(request).await?;
    //
    // println!("RESPONSE={:?}", response);
    //
    // Ok(())
}

// Cli init
